@@ -38,12 +38,15 @@
 
 static gint save_dialog(gint32 image_id, gint32 drawable);
 static void save_dialog_response(GtkWidget *widget, gint response_id, gpointer data);
-static void compression_selected(GtkWidget *widget, gpointer data);
-static void toggle_clicked(GtkWidget *widget, gpointer data);
 static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id);
 
 static int runme = 0;
 
+enum
+{
+   COMBO_VALUE, COMBO_STRING, COMBO_SENSITIVE
+};
+
 static const char *cubemap_face_names[4][6] =
 {
    {
@@ -78,13 +81,14 @@ static GtkWidget *format_opt;
 static GtkWidget *color_type_opt;
 static GtkWidget *dither_chk;
 static GtkWidget *mipmap_filter_opt;
-static GtkListStore *mipmap_filter_store;
 
-static struct
+typedef struct string_value_s
 {
-   int compression;
+   int value;
    char *string;
-} compression_strings[] =
+} string_value_t;
+
+static string_value_t compression_strings[] =
 {
    {DDS_COMPRESS_NONE,   "None"},
    {DDS_COMPRESS_BC1,    "BC1 / DXT1"},
@@ -99,11 +103,7 @@ static struct
    {-1, 0}
 };
 
-static struct
-{
-   int format;
-   char *string;
-} format_strings[] =
+static string_value_t format_strings[] =
 {
    {DDS_FORMAT_DEFAULT, "Default"},
    {DDS_FORMAT_RGB8, "RGB8"},
@@ -118,15 +118,12 @@ static struct
    {DDS_FORMAT_A8, "A8"},
    {DDS_FORMAT_L8, "L8"},
    {DDS_FORMAT_L8A8, "L8A8"},
+   {DDS_FORMAT_AEXP, "AExp"},
    {DDS_FORMAT_YCOCG, "YCoCg"},
    {-1, 0}
 };
 
-static struct
-{
-   int type;
-   char *string;
-} color_type_strings[] =
+static string_value_t color_type_strings[] =
 {
    {DDS_COLOR_DEFAULT,    "Default"},
    {DDS_COLOR_DISTANCE,   "Distance"},
@@ -135,11 +132,7 @@ static struct
    {-1, 0}
 };
 
-static struct
-{
-   int type;
-   char *string;
-} mipmap_filter_strings[] =
+static string_value_t mipmap_filter_strings[] =
 {
    {DDS_MIPMAP_DEFAULT,  "Default"},
    {DDS_MIPMAP_NEAREST,  "Nearest"},
@@ -150,6 +143,41 @@ static struct
    {-1, 0}
 };
 
+static string_value_t save_type_strings[] =
+{
+   {DDS_SAVE_SELECTED_LAYER, "Selected layer"},
+   {DDS_SAVE_CUBEMAP,        "As cube map"},
+   {DDS_SAVE_VOLUMEMAP,      "As volume map"},
+   {-1, 0}
+};
+
+static struct
+{
+   int format;
+   int bpp;
+   int alpha;
+   unsigned int rmask;
+   unsigned int gmask;
+   unsigned int bmask;
+   unsigned int amask;
+} format_info[] =
+{
+   {DDS_FORMAT_RGB8,    3, 0, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000},
+   {DDS_FORMAT_RGBA8,   4, 1, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000},
+   {DDS_FORMAT_BGR8,    3, 0, 0x000000ff, 0x0000ff00, 0x00ff0000, 0x00000000},
+   {DDS_FORMAT_ABGR8,   4, 1, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000},
+   {DDS_FORMAT_R5G6B5,  2, 0, 0x0000f800, 0x000007e0, 0x0000001f, 0x00000000},
+   {DDS_FORMAT_RGBA4,   2, 1, 0x00000f00, 0x000000f0, 0x0000000f, 0x0000f000},
+   {DDS_FORMAT_RGB5A1,  2, 1, 0x00007c00, 0x000003e0, 0x0000001f, 0x00008000},
+   {DDS_FORMAT_RGB10A2, 4, 1, 0x3ff00000, 0x000ffc00, 0x000003ff, 0xc0000000},
+   {DDS_FORMAT_R3G3B2,  1, 0, 0x000000e0, 0x0000001c, 0x00000003, 0x00000000},
+   {DDS_FORMAT_A8,      1, 0, 0x00000000, 0x00000000, 0x00000000, 0x000000ff},
+   {DDS_FORMAT_L8,      1, 0, 0x000000ff, 0x000000ff, 0x000000ff, 0x00000000},
+   {DDS_FORMAT_L8A8,    2, 1, 0x000000ff, 0x000000ff, 0x000000ff, 0x0000ff00},
+   {DDS_FORMAT_AEXP,    4, 1, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000},
+   {DDS_FORMAT_YCOCG,   4, 1, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000}
+};
+
 static int check_cubemap(gint32 image_id)
 {
    gint *layers, num_layers;
@@ -539,8 +567,8 @@ static void convert_pixels(unsigned char *dst, unsigned char *src,
             int co = TO_YCOCG_CO(r, g, b) + 128;
             int cg = TO_YCOCG_CG(r, g, b) + 128;
             dst[4 * i + 0] = a;
-            dst[4 * i + 1] = (cg < 0 ? 0 : (cg > 255 ? 255 : cg));
-            dst[4 * i + 2] = (co < 0 ? 0 : (co > 255 ? 255 : co));
+            dst[4 * i + 1] = MAX(0, MIN(255, cg));
+            dst[4 * i + 2] = MAX(0, MIN(255, co));
             dst[4 * i + 3] = TO_YCOCG_Y(r, g, b);
             break;
          }
@@ -653,8 +681,8 @@ static void convert_volume_pixels(unsigned char *dst, unsigned char *src,
             int co = TO_YCOCG_CO(r, g, b) + 128;
             int cg = TO_YCOCG_CG(r, g, b) + 128;
             dst[4 * i + 0] = a;
-            dst[4 * i + 1] = (cg < 0 ? 0 : (cg > 255 ? 255 : cg));
-            dst[4 * i + 2] = (co < 0 ? 0 : (co > 255 ? 255 : co));
+            dst[4 * i + 1] = MAX(0, MIN(255, cg));
+            dst[4 * i + 2] = MAX(0, MIN(255, co));
             dst[4 * i + 3] = TO_YCOCG_Y(r, g, b);
             break;
          }
@@ -954,7 +982,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
    unsigned char hdr[DDS_HEADERSIZE];
    unsigned int flags = 0, pflags = 0, caps = 0, caps2 = 0, size = 0;
    unsigned int rmask = 0, gmask = 0, bmask = 0, amask = 0;
-   char *format = "XXXX";
+   unsigned int fourcc = 0;
    gint32 num_layers, *layers;
    guchar *cmap;
    gint colors;
@@ -1002,111 +1030,18 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
    
    if(dds_write_vals.format > DDS_FORMAT_DEFAULT)
    {
-      switch(dds_write_vals.format)
+      for(i = 0; ; ++i)
       {
-         case DDS_FORMAT_RGB8:
-            fmtbpp = 3;
-            rmask = 0x00ff0000;
-            gmask = 0x0000ff00;
-            bmask = 0x000000ff;
-            amask = 0xff000000;
-            break;
-         case DDS_FORMAT_RGBA8:
-            fmtbpp = 4;
-            has_alpha = 1;
-            rmask = 0x00ff0000;
-            gmask = 0x0000ff00;
-            bmask = 0x000000ff;
-            amask = 0xff000000;
-            break;
-         case DDS_FORMAT_BGR8:
-            fmtbpp = 3;
-            rmask = 0x000000ff;
-            gmask = 0x0000ff00;
-            bmask = 0x00ff0000;
-            amask = 0x00000000;
-            break;
-         case DDS_FORMAT_ABGR8:
-            fmtbpp = 4;
-            has_alpha = 1;
-            rmask = 0x000000ff;
-            gmask = 0x0000ff00;
-            bmask = 0x00ff0000;
-            amask = 0xff000000;
-            break;
-         case DDS_FORMAT_R5G6B5:
-            fmtbpp = 2;
-            rmask = 0x0000f800;
-            gmask = 0x000007e0;
-            bmask = 0x0000001f;
-            amask = 0x00000000;
-            break;
-         case DDS_FORMAT_RGBA4:
-            fmtbpp = 2;
-            has_alpha = 1;
-            rmask = 0x00000f00;
-            gmask = 0x000000f0;
-            bmask = 0x0000000f;
-            amask = 0x0000f000;
-            break;
-         case DDS_FORMAT_RGB5A1:
-            fmtbpp = 2;
-            has_alpha = 1;
-            rmask = 0x00007c00;
-            gmask = 0x000003e0;
-            bmask = 0x0000001f;
-            amask = 0x00008000;
-            break;
-         case DDS_FORMAT_RGB10A2:
-            fmtbpp = 4;
-            has_alpha = 1;
-            rmask = 0x3ff00000;
-            gmask = 0x000ffc00;
-            bmask = 0x000003ff;
-            amask = 0xc0000000;
-            break;
-         case DDS_FORMAT_R3G3B2:
-            fmtbpp = 1;
-            has_alpha = 0;
-            rmask = 0x000000e0;
-            gmask = 0x0000001c;
-            bmask = 0x00000003;
-            amask = 0x00000000;
-            break;
-         case DDS_FORMAT_A8:
-            fmtbpp = 1;
-            has_alpha = 0;
-            rmask = 0x00000000;
-            gmask = 0x00000000;
-            bmask = 0x00000000;
-            amask = 0x000000ff;
-            break;
-         case DDS_FORMAT_L8:
-            fmtbpp = 1;
-            has_alpha = 0;
-            rmask = 0x000000ff;
-            gmask = 0x000000ff;
-            bmask = 0x000000ff;
-            amask = 0x00000000;
-            break;
-         case DDS_FORMAT_L8A8:
-            fmtbpp = 2;
-            has_alpha = 1;
-            rmask = 0x000000ff;
-            gmask = 0x000000ff;
-            bmask = 0x000000ff;
-            amask = 0x0000ff00;
-            break;
-         case DDS_FORMAT_YCOCG:
-            fmtbpp = 4;
-            has_alpha = 1;
-            rmask = 0x00ff0000;
-            gmask = 0x0000ff00;
-            bmask = 0x000000ff;
-            amask = 0xff000000;
-            break;
-         default:
+         if(format_info[i].format == dds_write_vals.format)
+         {
+            fmtbpp = format_info[i].bpp;
+            has_alpha = format_info[i].alpha;
+            rmask = format_info[i].rmask;
+            gmask = format_info[i].gmask;
+            bmask = format_info[i].bmask;
+            amask = format_info[i].amask;
             break;
+         }
       }
    }
    else if(bpp == 1)
@@ -1181,7 +1116,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
    {
       flags |= DDSD_MIPMAPCOUNT;
       caps |= (DDSCAPS_COMPLEX | DDSCAPS_MIPMAP);
-      num_mipmaps = get_num_mipmaps(w, h, dds_write_vals.compression);
+      num_mipmaps = get_num_mipmaps(w, h);
    }
    else
       num_mipmaps = 1;
@@ -1254,17 +1189,22 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
       PUTL32(hdr + 80, DDPF_FOURCC);
       switch(dds_write_vals.compression)
       {
-         case DDS_COMPRESS_BC1:    format = "DXT1"; break;
-         case DDS_COMPRESS_BC2:    format = "DXT3"; break;
+         case DDS_COMPRESS_BC1:
+            fourcc = FOURCC('D','X','T','1'); break;
+         case DDS_COMPRESS_BC2:
+            fourcc = FOURCC('D','X','T','3'); break;
          case DDS_COMPRESS_BC3:
          case DDS_COMPRESS_BC3N:
          case DDS_COMPRESS_YCOCG:
          case DDS_COMPRESS_YCOCGS:
-         case DDS_COMPRESS_AEXP:   format = "DXT5"; break;
-         case DDS_COMPRESS_BC4:    format = "ATI1"; break;
-         case DDS_COMPRESS_BC5:    format = "ATI2"; break;
+         case DDS_COMPRESS_AEXP:
+            fourcc = FOURCC('D','X','T','5'); break;
+         case DDS_COMPRESS_BC4:
+            fourcc = FOURCC('A','T','I','1'); break;
+         case DDS_COMPRESS_BC5:
+            fourcc = FOURCC('A','T','I','2'); break;
       }
-      memcpy(hdr + 84, format, 4);
+      PUTL32(hdr + 84, fourcc);
 
       size = ((w + 3) >> 2) * ((h + 3) >> 2);
       if(dds_write_vals.compression == DDS_COMPRESS_BC1 ||
@@ -1331,6 +1271,97 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
    return(1);
 }
 
+static GtkWidget *string_value_combo_new(string_value_t *strings,
+                                         int active_value)
+{
+   GtkWidget *opt;
+   GtkCellRenderer *renderer;
+   GtkListStore *store;
+   GtkTreeIter iter;
+   int i, active = 0;
+   
+   store = gtk_list_store_new(3, G_TYPE_INT, G_TYPE_STRING, G_TYPE_BOOLEAN);
+   for(i = 0; strings[i].string; ++i)
+   {
+      if(strings[i].value == active_value) active = i;
+      gtk_list_store_append(store, &iter);
+      gtk_list_store_set(store, &iter,
+                         0, strings[i].value,
+                         1, strings[i].string,
+                         2, 1,
+                         -1);
+   }
+ 
+   renderer = gtk_cell_renderer_text_new();
+   
+   opt = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
+   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(opt), renderer, 1);
+   gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(opt), renderer,
+                                  "text", COMBO_STRING,
+                                  "sensitive", COMBO_SENSITIVE,
+                                  NULL);
+   
+   gtk_combo_box_set_active(GTK_COMBO_BOX(opt), active);
+
+   g_object_unref(store);
+   
+   return(opt);
+}
+
+static void string_value_combo_selected(GtkWidget *widget, gpointer data)
+{
+   int value;
+   GtkTreeIter iter;
+   GtkTreeModel *model;
+   
+   model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
+   gtk_combo_box_get_active_iter(GTK_COMBO_BOX(widget), &iter);
+   gtk_tree_model_get(model, &iter, COMBO_VALUE, &value, -1);
+
+   *((int *)data) = value;
+}
+
+static void string_value_combo_set_item_sensitive(GtkWidget *widget,
+                                                  int value, int sensitive)
+{
+   GtkTreeIter iter;
+   GtkTreeModel *model;
+   int val;
+   
+   model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
+   gtk_tree_model_get_iter_first(model, &iter);
+   do
+   {
+      gtk_tree_model_get(model, &iter, COMBO_VALUE, &val, -1);
+      if(val == value)
+      {
+         gtk_list_store_set(GTK_LIST_STORE(model), &iter,
+                            COMBO_SENSITIVE, sensitive, -1);
+         break;
+      }
+   } while(gtk_tree_model_iter_next(model, &iter));
+}
+
+static void string_value_combo_set_active(GtkWidget *widget,
+                                          int value)
+{
+   GtkTreeIter iter;
+   GtkTreeModel *model;
+   int val;
+   
+   model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
+   gtk_tree_model_get_iter_first(model, &iter);
+   do
+   {
+      gtk_tree_model_get(model, &iter, COMBO_VALUE, &val, -1);
+      if(val == value)
+      {
+         gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widget), &iter);
+         break;
+      }
+   } while(gtk_tree_model_iter_next(model, &iter));
+}
+
 static void save_dialog_response(GtkWidget *widget, gint response_id,
                                  gpointer data)
 {
@@ -1346,7 +1377,13 @@ static void save_dialog_response(GtkWidget *widget, gint response_id,
 
 static void compression_selected(GtkWidget *widget, gpointer data)
 {
-   dds_write_vals.compression = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+   GtkTreeIter iter;
+   GtkTreeModel *model;
+   model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
+   gtk_combo_box_get_active_iter(GTK_COMBO_BOX(widget), &iter);
+   gtk_tree_model_get(model, &iter, COMBO_VALUE,
+                      &dds_write_vals.compression, -1);
+
    gtk_widget_set_sensitive(format_opt, dds_write_vals.compression == DDS_COMPRESS_NONE);
    gtk_widget_set_sensitive(color_type_opt,
                             dds_write_vals.compression != DDS_COMPRESS_NONE &&
@@ -1362,38 +1399,30 @@ static void compression_selected(GtkWidget *widget, gpointer data)
 
 static void savetype_selected(GtkWidget *widget, gpointer data)
 {
-   GtkTreeIter iter;
-   
    dds_write_vals.savetype = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
    
-   gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(mipmap_filter_store),
-                                 &iter, 0, DDS_MIPMAP_LANCZOS);
-      
    switch(dds_write_vals.savetype)
    {
-      case 0:
-      case 1:
+      case DDS_SAVE_SELECTED_LAYER:
+      case DDS_SAVE_CUBEMAP:
          gtk_widget_set_sensitive(compress_opt, 1);
-         gtk_list_store_set(mipmap_filter_store, &iter, 1, 1, -1);
+         string_value_combo_set_item_sensitive(mipmap_filter_opt,
+                                               DDS_MIPMAP_LANCZOS, 1);
          break;
-      case 2:
+      case DDS_SAVE_VOLUMEMAP:
          dds_write_vals.compression = DDS_COMPRESS_NONE;
          gtk_combo_box_set_active(GTK_COMBO_BOX(compress_opt), DDS_COMPRESS_NONE);
          gtk_widget_set_sensitive(compress_opt, 0);
          if(dds_write_vals.mipmap_filter == DDS_MIPMAP_LANCZOS)
             dds_write_vals.mipmap_filter = DDS_MIPMAP_DEFAULT;
-         gtk_combo_box_set_active(GTK_COMBO_BOX(mipmap_filter_opt),
-                                  dds_write_vals.mipmap_filter);
-         gtk_list_store_set(mipmap_filter_store, &iter, 1, 0, -1);
+         string_value_combo_set_active(mipmap_filter_opt,
+                                       dds_write_vals.mipmap_filter);
+         string_value_combo_set_item_sensitive(mipmap_filter_opt,
+                                               DDS_MIPMAP_LANCZOS, 0);
          break;
    }
 }
 
-static void format_selected(GtkWidget *widget, gpointer data)
-{
-   dds_write_vals.format = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-}
-
 static void toggle_clicked(GtkWidget *widget, gpointer data)
 {
    int *flag = (int*)data;
@@ -1433,16 +1462,6 @@ static void adv_opt_expanded(GtkWidget *widget, gpointer data)
    dds_write_vals.show_adv_opt = !gtk_expander_get_expanded(GTK_EXPANDER(widget));
 }
 
-static void color_type_selected(GtkWidget *widget, gpointer data)
-{
-   dds_write_vals.color_type = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-}
-
-static void mipmap_filter_selected(GtkWidget *widget, gpointer data)
-{
-   dds_write_vals.mipmap_filter = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-}
-
 static gint save_dialog(gint32 image_id, gint32 drawable_id)
 {
    GtkWidget *dlg;
@@ -1453,11 +1472,8 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    GtkWidget *check;
    GtkWidget *spin;
    GtkWidget *expander;
-   GtkListStore *list_store;
-   GtkTreeIter iter;
-   GtkCellRenderer *renderer;
    GimpImageType type, basetype;
-   int i, w, h;
+   int w, h;
    
    if(is_cubemap)
       dds_write_vals.savetype = DDS_SAVE_CUBEMAP;
@@ -1472,8 +1488,6 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    w = gimp_image_width(image_id);
    h = gimp_image_height(image_id);
    
-   renderer = gtk_cell_renderer_text_new();
-   
    dlg = gimp_dialog_new("Save as DDS", "dds", NULL, GTK_WIN_POS_MOUSE,
                          gimp_standard_help_func, SAVE_PROC,
                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -1505,20 +1519,14 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
                     (GtkAttachOptions)(0), 0, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    
-   opt = gtk_combo_box_new_text();
+
+   opt = string_value_combo_new(compression_strings,
+                                dds_write_vals.compression);
    gtk_widget_show(opt);
    gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 0, 1,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND), 0, 0);
    
-   for(i = 0; compression_strings[i].string; ++i)
-   {
-      gtk_combo_box_append_text(GTK_COMBO_BOX(opt),
-                                compression_strings[i].string);
-   }
-   
-   gtk_combo_box_set_active(GTK_COMBO_BOX(opt), dds_write_vals.compression);
-
    gtk_signal_connect(GTK_OBJECT(opt), "changed",
                       GTK_SIGNAL_FUNC(compression_selected), 0);
    
@@ -1530,23 +1538,16 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-   
-   opt = gtk_combo_box_new_text();
+
+   opt = string_value_combo_new(format_strings, dds_write_vals.format);
    gtk_widget_show(opt);
    gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 1, 2,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND), 0, 0);
-   
-   for(i = 0; format_strings[i].string; ++i)
-   {
-      gtk_combo_box_append_text(GTK_COMBO_BOX(opt),
-                                format_strings[i].string);
-   }
-   
-   gtk_combo_box_set_active(GTK_COMBO_BOX(opt), dds_write_vals.format);
 
    gtk_signal_connect(GTK_OBJECT(opt), "changed",
-                      GTK_SIGNAL_FUNC(format_selected), 0);
+                      GTK_SIGNAL_FUNC(string_value_combo_selected),
+                      &dds_write_vals.format);
 
    gtk_widget_set_sensitive(opt, dds_write_vals.compression == DDS_COMPRESS_NONE);
    
@@ -1559,25 +1560,18 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
                     (GtkAttachOptions)(0), 0, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    
-   list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
-   gtk_list_store_insert_with_values(list_store, 0, 0, 0, "Selected layer", 1, 1, -1);
-   gtk_list_store_insert_with_values(list_store, 0, 1, 0, "As cube map", 1, is_cubemap, -1);
-   gtk_list_store_insert_with_values(list_store, 0, 2, 0, "As volume map", 1, is_volume, -1);
-   
-   opt = gtk_combo_box_new_with_model(GTK_TREE_MODEL(list_store));
-   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(opt), renderer, 1);
-   gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(opt), renderer,
-                                  "text", 0, "sensitive", 1, NULL);
+   opt = string_value_combo_new(save_type_strings, dds_write_vals.savetype);
    gtk_widget_show(opt);
    gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 2, 3,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND), 0, 0);
    
-   gtk_combo_box_set_active(GTK_COMBO_BOX(opt), dds_write_vals.savetype);
-   
    gtk_signal_connect(GTK_OBJECT(opt), "changed",
                       GTK_SIGNAL_FUNC(savetype_selected), 0);
-   
+
+   string_value_combo_set_item_sensitive(opt, DDS_SAVE_CUBEMAP, is_cubemap);
+   string_value_combo_set_item_sensitive(opt, DDS_SAVE_VOLUMEMAP, is_volume);
+
    gtk_widget_set_sensitive(opt, is_cubemap || is_volume);
    
    check = gtk_check_button_new_with_label("Generate mipmaps");
@@ -1591,7 +1585,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    if(is_volume && dds_write_vals.savetype == DDS_SAVE_VOLUMEMAP)
    {
       dds_write_vals.compression = DDS_COMPRESS_NONE;
-      gtk_combo_box_set_active(GTK_COMBO_BOX(compress_opt), DDS_COMPRESS_NONE);
+      string_value_combo_set_active(compress_opt, DDS_COMPRESS_NONE);
       gtk_widget_set_sensitive(compress_opt, 0);
    }
    
@@ -1652,22 +1646,15 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
                     (GtkAttachOptions)(0), 0, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    
-   opt = gtk_combo_box_new_text();
+   opt = string_value_combo_new(color_type_strings, dds_write_vals.color_type);
    gtk_widget_show(opt);
    gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 0, 1,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND), 0, 0);
    
-   for(i = 0; color_type_strings[i].string; ++i)
-   {
-      gtk_combo_box_append_text(GTK_COMBO_BOX(opt),
-                                color_type_strings[i].string);
-   }
-   
-   gtk_combo_box_set_active(GTK_COMBO_BOX(opt), dds_write_vals.color_type);
-   
    gtk_signal_connect(GTK_OBJECT(opt), "changed",
-                      GTK_SIGNAL_FUNC(color_type_selected), 0);
+                      GTK_SIGNAL_FUNC(string_value_combo_selected),
+                      &dds_write_vals.color_type);
    
    color_type_opt = opt;
 
@@ -1689,34 +1676,19 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
                     (GtkAttachOptions)(0), 0, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    
-   mipmap_filter_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
-
-   for(i = 0; mipmap_filter_strings[i].string; ++i)
-   {
-      gtk_list_store_append(mipmap_filter_store, &iter);
-      gtk_list_store_set(mipmap_filter_store, &iter,
-                         0, mipmap_filter_strings[i].string,
-                         1, 1, -1);
-      if(mipmap_filter_strings[i].type == DDS_MIPMAP_LANCZOS &&
-         is_volume && dds_write_vals.savetype == DDS_SAVE_VOLUMEMAP)
-      {
-         gtk_list_store_set(mipmap_filter_store, &iter, 1, 0, -1);
-      }
-   }
-   
-   opt = gtk_combo_box_new_with_model(GTK_TREE_MODEL(mipmap_filter_store));
-   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(opt), renderer, 1);
-   gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(opt), renderer,
-                                  "text", 0, "sensitive", 1, NULL);
+   opt = string_value_combo_new(mipmap_filter_strings,
+                                dds_write_vals.mipmap_filter);
    gtk_widget_show(opt);
    gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 2, 3,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND), 0, 0);
-   
-   gtk_combo_box_set_active(GTK_COMBO_BOX(opt), dds_write_vals.mipmap_filter);
+
+   if(is_volume && dds_write_vals.savetype == DDS_SAVE_VOLUMEMAP)
+      string_value_combo_set_item_sensitive(opt, DDS_MIPMAP_LANCZOS, 0);
    
    gtk_signal_connect(GTK_OBJECT(opt), "changed",
-                      GTK_SIGNAL_FUNC(mipmap_filter_selected), 0);
+                      GTK_SIGNAL_FUNC(string_value_combo_selected),
+                      &dds_write_vals.mipmap_filter);
    
    mipmap_filter_opt = opt;
    
