@@ -32,29 +32,17 @@
 typedef void (*mipmapfunc_t)(unsigned char *, int, int, unsigned char *, int, int, int);
 typedef void (*volmipmapfunc_t)(unsigned char *, int, int, int, unsigned char *, int, int, int, int);
 
-int get_num_mipmaps(int width, int height, int format)
+int get_num_mipmaps(int width, int height)
 {
    int w = width << 1;
    int h = height << 1;
    int n = 0;
    
-   if(format == DDS_COMPRESS_NONE)
+   while(w != 1 || h != 1)
    {
-      while(w != 1 || h != 1)
-      {
-         if(w > 1) w >>= 1;
-         if(h > 1) h >>= 1;
-         ++n;
-      }
-   }
-   else
-   {
-      while(w > 4 && h > 4)
-      {
-         if(w > 4) w >>= 1;
-         if(h > 4) h >>= 1;
-         ++n;
-      }
+      if(w > 1) w >>= 1;
+      if(h > 1) h >>= 1;
+      ++n;
    }
    
    return(n);
@@ -63,13 +51,13 @@ int get_num_mipmaps(int width, int height, int format)
 unsigned int get_mipmapped_size(int width, int height, int bpp,
                                 int level, int num, int format)
 {
-   int w, h, n = 0;
+   int w, h, cw, ch, n = 0;
    unsigned int size = 0;
    
    w = width >> level;
    h = height >> level;
-   if(w == 0) w = 1;
-   if(h == 0) h = 1;
+   w = MAX(1, w);
+   h = MAX(1, h);
    w <<= 1;
    h <<= 1;
    
@@ -80,7 +68,11 @@ unsigned int get_mipmapped_size(int width, int height, int bpp,
       if(format == DDS_COMPRESS_NONE)
          size += (w * h);
       else
-         size += ((w + 3) >> 2) * ((h + 3) >> 2);
+      {
+         cw = MAX(w, 4);
+         ch = MAX(h, 4);
+         size += ((cw + 3) >> 2) * ((ch + 3) >> 2);
+      }
       ++n;
    }
    
@@ -107,9 +99,9 @@ unsigned int get_volume_mipmapped_size(int width, int height,
    w = width >> level;
    h = height >> level;
    d = depth >> level;
-   if(w == 0) w = 1;
-   if(h == 0) h = 1;
-   if(d == 0) d = 1;
+   w = MAX(1, w);
+   h = MAX(1, h);
+   d = MAX(1, d);
    w <<= 1;
    h <<= 1;
    d <<= 1;
@@ -472,10 +464,8 @@ int generate_mipmaps(unsigned char *dst, unsigned char *src,
    
    for(i = 1; i < mipmaps; ++i)
    {
-      dw = sw >> 1;
-      dh = sh >> 1;
-      if(dw < 1) dw = 1;
-      if(dh < 1) dh = 1;
+      dw = MAX(1, sw >> 1);
+      dh = MAX(1, sh >> 1);
   
       mipmap_func(d, dw, dh, s, sw, sh, bpp);
 
@@ -861,12 +851,9 @@ int generate_volume_mipmaps(unsigned char *dst, unsigned char *src,
    
    for(i = 1; i < mipmaps; ++i)
    {
-      dw = sw >> 1;
-      dh = sh >> 1;
-      dd = sd >> 1;
-      if(dw < 1) dw = 1;
-      if(dh < 1) dh = 1;
-      if(dd < 1) dd = 1;
+      dw = MAX(1, sw >> 1);
+      dh = MAX(1, sh >> 1);
+      dd = MAX(1, sd >> 1);
 
       mipmap_func(d, dw, dh, dd, s, sw, sh, sd, bpp);
 
